import click

from .commands import (
    add_post,
    get_posts,
    get_post,
    authenticate,
)


@click.group(name="wordpress")
def main():
    pass


main.add_command(authenticate)
main.add_command(get_posts)
main.add_command(get_post)
main.add_command(add_post)



import re

from ..settings import HTTP_SCHEMES


def make_url(url, suffix, *args):
    # remove any '://' from the start of the string.
    url = re.sub(r"^(:?\/+)", "", url)
    # remove any / from the end of the string.
    url = re.sub(r"(\/+)$", "", url)
    # if the url doesnt start with 'http://'' or 'https://' add 'https://'.
    if not url.startswith(HTTP_SCHEMES):
        url = "{}{}".format(HTTP_SCHEMES[1], url)
    # if the url doesn't end with the provided suffix, add it.
    if not url.endswith(suffix):
        url = "/".join([url, suffix])
    # if there are args also provided, add them to the end.
    url = "/".join([url] + list(args))
    return url



import json
from json import JSONDecodeError
import os

from ..settings import DEFAULT_CONFIG_FILE_NAME
from ..exceptions import ReadConfigException


def read_config(site_url, config_file_name=DEFAULT_CONFIG_FILE_NAME):
    path = os.path.join(os.path.expanduser("~"), config_file_name)
    if not os.path.isfile(path):
        raise ReadConfigException("Config file not found.")
    with open(path, "r", encoding="utf-8") as f:
        try:
            data = json.loads(f.read())
        except JSONDecodeError:
            data = {}
    if site_url not in data:
        raise ReadConfigException("There is not config for {}".format(site_url))
    return data[site_url]




import click
from slugify import slugify

from ....utils import (
    make_request,
    make_url,
)
from ....utils.wordpress import (
    create_post_content,
    read_credentials,
    get_authentication_header,
)
from ....settings import WORDPRESS_POSTS_URL


@click.command()
@click.option("--title")
@click.option("--markup")
@click.option("--status", default="publish")
@click.pass_obj
def add_post(options, title, markup, status):
    site_url = options.url
    timeout = options.timeout
    rest_url = make_url(site_url, WORDPRESS_POSTS_URL)
    last_post = make_request(method="get", url=rest_url, timeout=timeout)[0]
    click.echo(make_request(
        method="post",
        url=rest_url,
        timeout=timeout,
        headers=get_authentication_header(site_url),
        data={
            #"date": "",
            #"date_gmt", "",
            "slug": slugify(title),
            "status": status,
            #"password": None,
            "title": title,
            "content": create_post_content(site_url, markup),
            #"author": 3,
            #"excerpt": last_post["excerpt"],
            #"featured_media": last_post["featured_media"],
            #"comment_status": last_post["comment_status"],
            #"ping_status": last_post["ping_status"],
            #"format": last_post["format"],
            "meta": last_post["meta"],
            #"sticky": last_post["sticky"],
            "template": last_post["template"],
        },
    ))



import json
from json import JSONDecodeError
import os

from ..read_config import read_config
from ...exceptions import (
    ReadWordpressJWTConfigException,
    ReadWordpressConfigException
)


def read_jwt(site_url, config_file_name=None):
    config = read_config(site_url, config_file_name)
    wordpress = config.get("wordpress", None)
    if wordpress is None:
        raise ReadWordpressConfigException(
            "No wordpress config for {}".format(site_url)
        )
    jwt = wordpress.get("jwt", None)
    if jwt is None:
        raise ReadWordpressJWTConfigException(
            "No wordpress JWT config for {}".format(site_url)
        )
    return jwt



def get_header_html(html, closing_tag="</p>"):
    closing_tag_start = html.find(closing_tag)
    closing_tag_end = closing_tag_start + len(closing_tag)
    result = html[0:closing_tag_end]
    return result



import os
import json
from json import JSONDecodeError

from ..settings import DEFAULT_CONFIG_FILE_NAME


def save_config(
    site_url,
    key,
    value,
    config_file_name=DEFAULT_CONFIG_FILE_NAME
):
    config_file = os.path.join(os.path.expanduser("~"), config_file_name)
    encoding = "utf-8"
    data = {}
    # Get the existing data for the site.
    if os.path.exists(config_file):
        with open(config_file, "r", encoding=encoding) as old_file:
            try:
                data = json.loads(old_file.read())
            except JSONDecodeError:
                data = {}
    # Update the data with our passed in value.
    with open(config_file, "w", encoding=encoding) as new_file:
        # get the data for the site.
        entry = data.get(site_url, {})
        # update the data using the new value
        entry.update({key: value})
        # set the data back into the original data.
        data[site_url] = entry
        # write the file.
        new_file.write(json.dumps(data))




WORDPRESS_URL_PREFIX = "wp-json"

WORDPRESS_POSTS_URL = "{}/wp/v2/posts".format(WORDPRESS_URL_PREFIX)

WORDPRESS_JWT_URL = "{}/jwt-auth/v1".format(WORDPRESS_URL_PREFIX)

DEFAULT_REQUEST_TIMEOUT = 10.0

DEFAULT_REQUEST_HEADERS = {
    "Content-Type": "application/json",
    "Accept": "application/json",
}

DEFAULT_CONFIG_FILE_NAME = "birdhouse_toolbox.json"

HTTP_SCHEMES = ("http://", "https://",)

HTTP_METHODS_ALLOWED = ("get", "post", "put", "delete", "options", "head",)



import click

from ....utils import make_request, make_url
from ....utils.wordpress import read_credentials
from ....settings import WORDPRESS_POSTS_URL


@click.command()
@click.pass_obj
def get_posts(options):
    headers = {}
    click.echo(make_request(
        method="get",
        url=make_url(options.url, WORDPRESS_POSTS_URL),
        headers=headers,
        timeout=options.timeout,
    ))



from .create_post_content import create_post_content
from .get_post_content import get_post_content
from .get_footer_html import get_footer_html
from .get_form_html import get_form_html
from .get_header_html import get_header_html
from .get_tag_html import get_tag_html
from .read_credentials import read_credentials
from .save_credentials import save_credentials
from .get_authentication_header import get_authentication_header
from .format_as_html import format_as_html
from .get_jwt import get_jwt
from .save_jwt import save_jwt
from .read_jwt import read_jwt



import os
import json
from json import JSONDecodeError

from ..save_config import save_config
from ...settings import DEFAULT_CONFIG_FILE_NAME


def save_credentials(
    site_url,
    username,
    password,
    config_file_name=DEFAULT_CONFIG_FILE_NAME
):
    payload = {"username": username, "password": password}
    save_config(site_url, "wordpress", {"credentials": payload}, config_file_name)



class RequestMethodIncorrectException(Exception):
    pass


class RequestTimeoutException(Exception):
    pass


class RequestFailureException(Exception):
    pass


class ReadConfigException(Exception):
    pass


class ReadWordpressConfigException(Exception):
    pass


class ReadWordpressCredentialsConfigException(Exception):
    pass


class ReadWordpressJWTConfigException(Exception):
    pass



import click

from .options import Options
from .wordpress import main as wordpress_main
from .analytics import main as analytics_main
from ..settings import DEFAULT_REQUEST_TIMEOUT


@click.group()
@click.option("--url")
@click.option("--timeout", default=DEFAULT_REQUEST_TIMEOUT, required=False)
@click.pass_context
def main(ctx, url, timeout):
    ctx.obj = Options(url, timeout)


main.add_command(wordpress_main)
main.add_command(analytics_main)





def get_footer_html(html, opening_tag="<p>"):
    opening_tag_start = html.rfind(opening_tag)
    opening_tag_end = opening_tag_start + len(opening_tag)
    closing_tag = opening_tag.replace("<", "</")
    closing_tag_start = html.rfind(closing_tag)
    closing_tag_end = closing_tag_start + len(closing_tag)
    result = html[opening_tag_start:closing_tag_end]
    return result



import click

from ....utils.wordpress import (
    save_credentials,
    get_jwt,
    save_jwt,
    read_credentials
)


@click.command()
@click.option("--username", default=None, required=False)
@click.option("--password", default=None, required=False)
@click.pass_obj
def authenticate(options, username, password):
    url = options.url
    if username is None and password is None:
        credentials = read_credentials(url)
        username = credentials["username"]
        password = credentials["password"]
    else:
        save_credentials(url, username, password)
    save_jwt(url, get_jwt(url, username, password))



from ..make_request import make_request
from ..make_url import make_url
from ...settings import WORDPRESS_JWT_URL


def get_jwt(site_url, username, password):
    full_url = make_url(site_url, WORDPRESS_JWT_URL, "token")
    headers = {
        "Content-Type": "application/x-www-form-urlencoded",
    }
    data = {
        "username": username,
        "password": password,
    }
    return make_request(
        method="post",
        url=full_url,
        headers=headers,
        data=data,
    )



import json
from json import JSONDecodeError
import os

from ..read_config import read_config
from ...exceptions import (
    ReadWordpressConfigException,
    ReadWordpressCredentialsConfigException,
)


def read_credentials(site_url, config_file_name=None):
    config = read_config(site_url, config_file_name)
    wordpress = config.get("wordpress", None)
    if wordpress is None:
        raise ReadWordpressConfigException(
            "No wordpress config for {}".format(site_url)
        )
    credentials = wordpress.get("credentials", None)
    if credentials is None:
        raise ReadWordpressCredentialsConfigException(
           "No wordpress credentials config for {}.".format(site_url)
        )
    return credentials



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages
from setup_utils import PACKAGE_NAME, read


setup(
    name=PACKAGE_NAME,
    version=read(("src", PACKAGE_NAME, "__init__.py",), "__version__"),
    description=(
        "A collection of commnad line tools to automate various online "
        "marketing processes"
    ),
    long_description=read(("README.md",)),
    long_description_content_type="text/markdown",
    author="Alex Seitsinger",
    author_email="contact@alexseitsinger.com",
    url="https://github.com/alexseitsinger/{}".format(PACKAGE_NAME),
    install_requires=["requests", "click", "python-slugify"],
    entry_points={"console_scripts": ["bht={}.cli:main".format(PACKAGE_NAME)]},
    package_dir={"": "src"},
    packages=find_packages("src", exclude=["tests"]),
    include_package_data=True,
    license="BSD 2-Clause License",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
    ]
)



import requests
import json

from ..settings import (
    DEFAULT_REQUEST_TIMEOUT,
    DEFAULT_REQUEST_HEADERS,
    HTTP_METHODS_ALLOWED,
)
from ..exceptions import (
    RequestMethodIncorrectException,
    RequestTimeoutException,
    RequestFailureException,
)


def make_request(
    method,
    url,
    params={},
    data={},
    headers={},
    timeout=DEFAULT_REQUEST_TIMEOUT,
):
    # get the request method to use.
    method = method.lower()
    try:
        fn = getattr(requests, method)
    except AttributeError:
        raise RequestMethodIncorrectException(
            "Method must be one of: {}".format(
                ", ".join(list(HTTP_METHODS_ALLOWED))
            )
        )
    # Create a dictionary of the final headers to use.
    final_headers = {}
    final_headers.update(DEFAULT_REQUEST_HEADERS)
    final_headers.update(headers)
    # Try to make a successful request...
    try:
        # If its a "get" method, dont append data to it. Only params.
        if method in ("get", "options", "head",):
            response = fn(
                url,
                params=params,
                headers=final_headers,
                timeout=timeout,
            )
        else:
            # Otherwise, attempt to encode the data if its supposed to be JSON.
            current_content_type = final_headers["Content-Type"]
            json_content_type = "application/json"
            if current_content_type == json_content_type:
                data = json.dumps(data)
            response = fn(
                url,
                params=params,
                data=data,
                headers=final_headers,
                timeout=timeout,
            )
        status_code = response.status_code
        content = response.json()
        if not str(status_code).startswith("2"):
            raise RequestFailureException(
                "{}: {} - {}".format(url, status_code, content["message"])
            )
        return content
    except requests.ConnectionError:
        raise RequestTimeoutException(
            "{}: Timed out after {} seconds.".format(url, timeout)
            )



from ..make_url import make_url
from ..make_request import make_request
from ...settings import WORDPRESS_POSTS_URL, DEFAULT_REQUEST_TIMEOUT


def get_post_content(site_url, post_number=0):
    rest_url = make_url(site_url, WORDPRESS_POSTS_URL)
    response = make_request("get", url=rest_url, timeout=DEFAULT_REQUEST_TIMEOUT)
    return response[post_number]["content"]["rendered"]



import os
import json
from json import JSONDecodeError

from ..save_config import save_config
from ...settings import DEFAULT_CONFIG_FILE_NAME


def save_jwt(site_url, payload, config_file_name=DEFAULT_CONFIG_FILE_NAME):
    save_config(site_url, "wordpress", {"jwt": payload}, config_file_name)
